@@ -17,7 +17,7 @@ namespace internal {
 StubCache::StubCache(Isolate* isolate) : isolate_(isolate) {
   // Ensure the nullptr (aka Smi::zero()) which StubCache::Get() returns
   // when the entry is not found is not considered as a handler.
-  DCHECK(!IC::IsHandler(MaybeObject()));
+  DCHECK(!IC::IsHandler(Tagged<MaybeObject>()));
 }
 
 void StubCache::Initialize() {
@@ -67,31 +67,34 @@ int StubCache::SecondaryOffsetForTesting(Tagged<Name> name, Tagged<Map> map) {
 namespace {
 
 bool CommonStubCacheChecks(StubCache* stub_cache, Tagged<Name> name,
-                           Tagged<Map> map, MaybeObject handler) {
+                           Tagged<Map> map, Tagged<MaybeObject> handler) {
   // Validate that the name and handler do not move on scavenge, and that we
   // can use identity checks instead of structural equality checks.
   DCHECK(!Heap::InYoungGeneration(name));
   DCHECK(!Heap::InYoungGeneration(handler));
   DCHECK(IsUniqueName(name));
-  if (handler->ptr() != kNullAddress) DCHECK(IC::IsHandler(handler));
+  if (handler.ptr() != kNullAddress) DCHECK(IC::IsHandler(handler));
   return true;
 }
 
 }  // namespace
 #endif
 
-void StubCache::Set(Tagged<Name> name, Tagged<Map> map, MaybeObject handler) {
+void StubCache::Set(Tagged<Name> name, Tagged<Map> map,
+                    Tagged<MaybeObject> handler) {
   DCHECK(CommonStubCacheChecks(this, name, map, handler));
 
   // Compute the primary entry.
   int primary_offset = PrimaryOffset(name, map);
   Entry* primary = entry(primary_, primary_offset);
-  MaybeObject old_handler(
+  Tagged<MaybeObject> old_handler(
       TaggedValue::ToMaybeObject(isolate(), primary->value));
   // If the primary entry has useful data in it, we retire it to the
   // secondary cache before overwriting it.
-  if (old_handler != MaybeObject::FromObject(
-                         isolate()->builtins()->code(Builtin::kIllegal)) &&
+  // We need SafeEquals here while Builtin Code objects still live in the RO
+  // space inside the sandbox.
+  static_assert(!kAllCodeObjectsLiveInTrustedSpace);
+  if (!old_handler.SafeEquals(isolate()->builtins()->code(Builtin::kIllegal)) &&
       !primary->map.IsSmi()) {
     Tagged<Map> old_map =
         Map::cast(StrongTaggedValue::ToObject(isolate(), primary->map));
@@ -109,8 +112,8 @@ void StubCache::Set(Tagged<Name> name, Tagged<Map> map, MaybeObject handler) {
   isolate()->counters()->megamorphic_stub_cache_updates()->Increment();
 }
 
-MaybeObject StubCache::Get(Tagged<Name> name, Tagged<Map> map) {
-  DCHECK(CommonStubCacheChecks(this, name, map, MaybeObject()));
+Tagged<MaybeObject> StubCache::Get(Tagged<Name> name, Tagged<Map> map) {
+  DCHECK(CommonStubCacheChecks(this, name, map, Tagged<MaybeObject>()));
   int primary_offset = PrimaryOffset(name, map);
   Entry* primary = entry(primary_, primary_offset);
   if (primary->key == name && primary->map == map) {
@@ -121,12 +124,11 @@ MaybeObject StubCache::Get(Tagged<Name> name, Tagged<Map> map) {
   if (secondary->key == name && secondary->map == map) {
     return TaggedValue::ToMaybeObject(isolate(), secondary->value);
   }
-  return MaybeObject();
+  return Tagged<MaybeObject>();
 }
 
 void StubCache::Clear() {
-  MaybeObject empty =
-      MaybeObject::FromObject(isolate_->builtins()->code(Builtin::kIllegal));
+  Tagged<MaybeObject> empty = isolate_->builtins()->code(Builtin::kIllegal);
   Tagged<Name> empty_string = ReadOnlyRoots(isolate()).empty_string();
   for (int i = 0; i < kPrimaryTableSize; i++) {
     primary_[i].key = StrongTaggedValue(empty_string);
