@@ -35,7 +35,7 @@ const {
   validateAbortSignal,
   validateObject,
   validateString,
-  validateInternalField,
+  validateThisInternalField,
   kValidateObjectAllowObjects,
 } = require('internal/validators');
 
@@ -1118,7 +1118,7 @@ function defineEventHandler(emitter, name, event = name) {
   // 8.1.5.1 Event handlers - basically `on[eventName]` attributes
   const propName = `on${name}`;
   function get() {
-    validateInternalField(this, kHandlers, 'EventTarget');
+    validateThisInternalField(this, kHandlers, 'EventTarget');
     return this[kHandlers]?.get(event)?.handler ?? null;
   }
   ObjectDefineProperty(get, 'name', {
@@ -1127,7 +1127,7 @@ function defineEventHandler(emitter, name, event = name) {
   });
 
   function set(value) {
-    validateInternalField(this, kHandlers, 'EventTarget');
+    validateThisInternalField(this, kHandlers, 'EventTarget');
     let wrappedHandler = this[kHandlers]?.get(event);
     if (wrappedHandler) {
       if (typeof wrappedHandler.handler === 'function') {
