@@ -1,16 +1,17 @@
 'use strict';
 
 const {
-  ArrayPrototypeMap,
+  ArrayPrototypePush,
   Boolean,
   FunctionPrototypeCall,
   JSONParse,
-  ObjectKeys,
+  ObjectAssign,
   ObjectPrototypeHasOwnProperty,
   ReflectApply,
   SafeArrayIterator,
   SafeMap,
   SafeSet,
+  SafeWeakMap,
   StringPrototypeIncludes,
   StringPrototypeReplaceAll,
   StringPrototypeSlice,
@@ -102,7 +103,8 @@ translators.set('module', function moduleStrategy(url, source, isMain) {
   source = stringify(source);
   debug(`Translating StandardModule ${url}`);
   const { compileSourceTextModule } = require('internal/modules/esm/utils');
-  const module = compileSourceTextModule(url, source, this);
+  const context = isMain ? { isMain } : undefined;
+  const module = compileSourceTextModule(url, source, this, context);
   return module;
 });
 
@@ -422,6 +424,14 @@ translators.set('json', function jsonStrategy(url, source) {
 });
 
 // Strategy for loading a wasm module
+// This logic should collapse into WebAssembly Module Record in future.
+/**
+ * @type {WeakMap<
+ *   import('internal/modules/esm/utils').ModuleNamespaceObject,
+ *   WebAssembly.Instance
+ * >} [[Instance]] slot proxy for WebAssembly Module Record
+ */
+const wasmInstances = new SafeWeakMap();
 translators.set('wasm', async function(url, source) {
   emitExperimentalWarning('Importing WebAssembly modules');
 
@@ -440,35 +450,78 @@ translators.set('wasm', async function(url, source) {
     throw err;
   }
 
-  const imports =
-      ArrayPrototypeMap(WebAssembly.Module.imports(compiled),
-                        ({ module }) => module);
-  const exports =
-    ArrayPrototypeMap(WebAssembly.Module.exports(compiled),
-                      ({ name }) => name);
-
-  const createDynamicModule = require(
-    'internal/modules/esm/create_dynamic_module');
-  return createDynamicModule(imports, exports, url, (reflect) => {
-    const { exports } = new WebAssembly.Instance(compiled, reflect.imports);
-    for (const expt of ObjectKeys(exports)) {
-      reflect.exports[expt].set(exports[expt]);
+  const importsList = new SafeSet();
+  const wasmGlobalImports = [];
+  for (const impt of WebAssembly.Module.imports(compiled)) {
+    if (impt.kind === 'global') {
+      ArrayPrototypePush(wasmGlobalImports, impt);
     }
-  }).module;
+    importsList.add(impt.module);
+  }
+
+  const exportsList = new SafeSet();
+  const wasmGlobalExports = new SafeSet();
+  for (const expt of WebAssembly.Module.exports(compiled)) {
+    if (expt.kind === 'global') {
+      wasmGlobalExports.add(expt.name);
+    }
+    exportsList.add(expt.name);
+  }
+
+  const createDynamicModule = require('internal/modules/esm/create_dynamic_module');
+
+  const { module } = createDynamicModule([...importsList], [...exportsList], url, (reflect) => {
+    for (const impt of importsList) {
+      const importNs = reflect.imports[impt];
+      const wasmInstance = wasmInstances.get(importNs);
+      if (wasmInstance) {
+        const wrappedModule = ObjectAssign({ __proto__: null }, reflect.imports[impt]);
+        for (const { module, name } of wasmGlobalImports) {
+          if (module !== impt) {
+            continue;
+          }
+          // Import of Wasm module global -> get direct WebAssembly.Global wrapped value.
+          // JS API validations otherwise remain the same.
+          wrappedModule[name] = wasmInstance[name];
+        }
+        reflect.imports[impt] = wrappedModule;
+      }
+    }
+    // In cycles importing unexecuted Wasm, wasmInstance will be undefined, which will fail during
+    // instantiation, since all bindings will be in the Temporal Deadzone (TDZ).
+    const { exports } = new WebAssembly.Instance(compiled, reflect.imports);
+    wasmInstances.set(module.getNamespace(), exports);
+    for (const expt of exportsList) {
+      let val = exports[expt];
+      // Unwrap WebAssembly.Global for JS bindings
+      if (wasmGlobalExports.has(expt)) {
+        try {
+          // v128 will throw in GetGlobalValue, see:
+          // https://webassembly.github.io/esm-integration/js-api/index.html#getglobalvalue
+          val = val.value;
+        } catch {
+          // v128 doesn't support ToJsValue() -> use undefined (ideally should stay in TDZ)
+          continue;
+        }
+      }
+      reflect.exports[expt].set(val);
+    }
+  });
+  return module;
 });
 
 // Strategy for loading a commonjs TypeScript module
-translators.set('commonjs-typescript', function(url, source) {
+translators.set('commonjs-typescript', function(url, source, isMain) {
   assertBufferSource(source, true, 'load');
   const code = stripTypeScriptModuleTypes(stringify(source), url);
   debug(`Translating TypeScript ${url}`);
-  return FunctionPrototypeCall(translators.get('commonjs'), this, url, code, false);
+  return FunctionPrototypeCall(translators.get('commonjs'), this, url, code, isMain);
 });
 
 // Strategy for loading an esm TypeScript module
-translators.set('module-typescript', function(url, source) {
+translators.set('module-typescript', function(url, source, isMain) {
   assertBufferSource(source, true, 'load');
   const code = stripTypeScriptModuleTypes(stringify(source), url);
   debug(`Translating TypeScript ${url}`);
-  return FunctionPrototypeCall(translators.get('module'), this, url, code, false);
+  return FunctionPrototypeCall(translators.get('module'), this, url, code, isMain);
 });
