@@ -7,7 +7,6 @@ const {
   AtomicsAdd,
   Float64Array,
   FunctionPrototypeBind,
-  JSONStringify,
   MathMax,
   ObjectEntries,
   Promise,
@@ -59,7 +58,7 @@ const {
 const { createMainThreadPort, destroyMainThreadPort } = require('internal/worker/messaging');
 const { deserializeError } = require('internal/error_serdes');
 const { fileURLToPath, isURL, pathToFileURL } = require('internal/url');
-const { kEmptyObject } = require('internal/util');
+const { kEmptyObject, SymbolAsyncDispose } = require('internal/util');
 const { validateArray, validateString } = require('internal/validators');
 const {
   throwIfBuildingSnapshot,
@@ -166,8 +165,8 @@ class Worker extends EventEmitter {
       doEval = 'classic';
     } else if (isURL(filename) && filename.protocol === 'data:') {
       url = null;
-      doEval = 'module';
-      filename = `import ${JSONStringify(`${filename}`)}`;
+      doEval = 'data-url';
+      filename = `${filename}`;
     } else {
       doEval = false;
       if (isURL(filename)) {
@@ -407,6 +406,10 @@ class Worker extends EventEmitter {
     });
   }
 
+  async [SymbolAsyncDispose]() {
+    await this.terminate();
+  }
+
   ref() {
     if (this[kHandle] === null) return;
 
