@@ -217,10 +217,11 @@ function pad(n) {
 }
 
 /**
- * @param {string} code
+ * @param {string} [code]
  * @returns {string}
  */
 function escapeStyleCode(code) {
+  if (code === undefined) return '';
   return `\u001b[${code}m`;
 }
 
@@ -256,6 +257,7 @@ function styleText(format, text, { validateStream = true, stream = process.stdou
   let left = '';
   let right = '';
   for (const key of formatArray) {
+    if (key === 'none') continue;
     const formatCodes = inspect.colors[key];
     // If the format is not a valid style, throw an error
     if (formatCodes == null) {
