@@ -1,11 +1,12 @@
 'use strict';
 
 const common = require('../common');
-if (!common.hasCrypto)
-  common.skip('missing crypto');
+if ((!common.hasCrypto) || (!common.hasIntl)) {
+  common.skip('ESLint tests require crypto and Intl');
+}
 common.skipIfEslintMissing();
 
-const RuleTester = require('../../tools/node_modules/eslint').RuleTester;
+const RuleTester = require('../../tools/eslint/node_modules/eslint').RuleTester;
 const rule = require('../../tools/eslint-rules/documented-errors');
 
 const invalidCode = 'UNDOCUMENTED ERROR CODE';
@@ -26,11 +27,6 @@ new RuleTester().run('documented-errors', rule, {
           message: `"${invalidCode}" is not documented in doc/api/errors.md`,
           line: 2
         },
-        {
-          message:
-            `doc/api/errors.md does not have an anchor for "${invalidCode}"`,
-          line: 2
-        },
       ]
     },
   ]
